#!/usr/bin/python

import os
import pwd
import sys
import subprocess
import base64
try:
    from jinja2 import Template
except ImportError:
    pass

from charmhelpers.core.hookenv import (
    Hooks,
    UnregisteredHookError,
    config,
    log,
    relation_get,
    relation_set,
    unit_private_ip,
)

from charmhelpers.fetch import (
    apt_install,
    apt_update,
)

from charmhelpers.contrib.openstack.utils import (
    configure_installation_source,
)

from charmhelpers.contrib.unison import (
    ensure_user,
    get_keypair,
)

from charmhelpers.payload.execd import execd_preinstall

default_nagios_plugin_dir = "/usr/lib/nagios/plugins"


hooks = Hooks()

BASE_PACKAGES = [
    'openvpn',
    'openssl',
    'python-jinja2',
]


# FIXME
def determine_packages():
    apt_extra_packages = config('apt_extra_packages')
    if apt_extra_packages:
        packages = [] + BASE_PACKAGES + apt_extra_packages
    else:
        packages = [] + BASE_PACKAGES
    return list(set(packages))


@hooks.hook('config-changed')
def config_changed():
    conf = config()

    os.umask(07)
    create_user(conf['user'], conf['group'])
    vars = Template(open("templates/vars.tmpl").read()).render(conf)
    dir_name = "{}/easyrsa3".format(conf['directory'])
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)
    file_name = "{}/vars".format(dir_name)
    with open(file_name, "w") as vars_file:
        vars_file.write(vars)
    subprocess.call(['chown', '{}:{}'.format(conf['user'], conf['group']), file_name])


def create_user(user, group):
    conf = config()
    ensure_user(user, group)
    home_dir = pwd.getpwnam(conf['user']).pw_dir
    if not os.path.exists(home_dir):
        os.makedirs(home_dir)
        subprocess.call(['chown', '{}:{}'.format(user, group), home_dir])
    ssh_dir = "{}/.ssh".format(home_dir)
    if not os.path.exists(ssh_dir):
        os.makedirs(ssh_dir)
        subprocess.call(['chown', '{}:{}'.format(user, group), ssh_dir])


@hooks.hook('install', 'upgrade-charm')
def install():
    conf = config()
    execd_preinstall()
    configure_installation_source(conf['openvpn_origin'])
    apt_update()
    apt_install(determine_packages(), fatal=True)
    create_user(conf['user'], conf['group'])
    if not os.path.exists(conf['directory']):
        os.makedirs(conf['directory'])
    os.chdir(conf['directory'])
    # create ssh key
    get_keypair(conf['user'])[1]
    if not os.path.exists('easyrsa3'):
        os.makedirs('easyrsa3')
    subprocess.call(['chown', '{}:{}'.format(conf['user'], conf['group']), 'easyrsa3'])

    old_uid = os.geteuid()
    # Unpack the zip from https://github.com/OpenVPN/easy-rsa into place
    os.seteuid(pwd.getpwnam(conf['user']).pw_uid)
    subprocess.call(['tar', 'xzf', '{}/files/easyrsa.tar.gz'.format(os.environ['CHARM_DIR'])])
    # now create the ca, and support files.
    os.chdir("easyrsa3")
    subprocess.call(['./easyrsa', 'init-pki'])
    open("pki/index.txt", "w")
    open("pki/serial", "w").write("01\n")
    os.environ['EASYRSA_REQ_CN'] = "{} CA".format(conf['req_org'])
    subprocess.call(['./easyrsa', '--batch', 'build-ca', 'nopass'])
    subprocess.call(['./easyrsa', 'gen-crl'])

    open("build-key", "w").write('#!/bin/bash\n./easyrsa gen-req "$1" nopass \n./easyrsa sign-req client "$1" nopass\n')
    os.chmod('build-key', 0555)
    open("build-key-server", "w").write('#!/bin/bash\n./easyrsa gen-req "$1" nopass \n./easyrsa sign-req server "$1" nopass\n')
    os.chmod('build-key-server', 0555)
    os.seteuid(old_uid)


@hooks.hook('openvpn-pki-recipient-relation-joined')
@hooks.hook('openvpn-pki-recipient-relation-changed')
def pki_recipient():
    conf = config()
    ca_cert = base64.b64encode(open("{}/easyrsa3/pki/ca.crt".format(conf['directory'])).read())
    relation_set(sync_key=get_keypair(conf['user'])[1], sync_ip=unit_private_ip(), ca_certificate=ca_cert)
    user_uid = pwd.getpwnam(conf['user']).pw_uid

    data = relation_get()
    hostname = data.get('hostname','').replace('/','_')
    mode = data.get('mode','')
    if hostname:
        # if there is no certificate, then generate one
        if not os.access('{}/easyrsa3/pki/issued/{}.crt'.format(conf['directory'],hostname),os.F_OK):
            old_uid = os.geteuid()
            os.seteuid(user_uid)
            os.chdir('{}/easyrsa3'.format(conf['directory']))
            if mode == 'server':
                subprocess.call(['build-key-server', hostname])
            else:
                subprocess.call(['build-key', hostname])
            os.seteuid(old_uid)
        # add the key/cert to the relation data
        server_key = base64.b64encode(open('pki/private/{}.key'.format(hostname)).read())
        server_certificate = base64.b64encode(open('pki/issued/{}.crt'.format(hostname)).read())
        relation_set(server_key=server_key, server_certificate=server_certificate, server_cn=hostname)


def main():
    try:
        hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log('Unknown hook {} - skipping.'.format(e))


if __name__ == '__main__':
    main()
